/*
    UHU-menu -- input plugin for freedesktop-style files
    Copyright (C) 2003  Koblinger Egmont <egmont@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <limits.h>

#include "uhu-menu.h"

static int path_remove;

static int build_menu (const char *dir, struct item *item)
{
	DIR *d;
	struct dirent *de;
	char buf[PATH_MAX];
	int l;
	struct item itemtmp;
	struct stat st;

	sprintf(buf, "%s/.directory", dir);
	if (desktop_parse(buf, path_remove, item) < 0) return -1;
	l = strlen(item->file_name);
	if (l >= 11 && !strcmp(item->file_name + l - 11, "/.directory")) {
		item->file_name[l-11] = '\0';
	}
	item->subitems_nr = 0;
	item->subitems = NULL;
	item->issubmenu = 1;

	d = opendir(dir);
	if (d == NULL) {
		message_e(3, "opendir failed");
		return -1;
	}
	while ((de = readdir(d)) != NULL) {
		if (de->d_name[0] == '.') continue;
		l = strlen(de->d_name);
		sprintf(buf, "%s/%s", dir, de->d_name);
		if (stat(buf, &st) < 0) continue;
		if (S_ISREG(st.st_mode) && l >= 8 && !strcmp(de->d_name + l - 8, ".desktop")) {
			if (desktop_parse(buf, path_remove, &itemtmp) < 0) continue;
			itemtmp.issubmenu = 0;
		} else if (S_ISDIR(st.st_mode)) {
			if (build_menu(buf, &itemtmp) < 0) continue;
		} else {
			continue;
		}
		if (item->subitems_nr == 0) {
			item->subitems = malloc(sizeof(struct item));
		} else {
			item->subitems = realloc(item->subitems, (item->subitems_nr + 1) * sizeof(struct item));
		}
		memcpy(&item->subitems[item->subitems_nr], &itemtmp, sizeof(struct item));
		item->subitems_nr++;
	}
	closedir(d);
	return 0;
}

struct item *read_menu (const char *path)
{
	char buf[PATH_MAX];
	struct item *item;

	sprintf(buf, "%s/menu-source", path);
	path_remove = strlen(buf);

	item = malloc(sizeof(struct item));
	if (build_menu(buf, item) < 0) return NULL;
	return item;
}
